@@ -14,12 +14,14 @@ typedef struct {
     STRLEN buf_len;
 
     U32 flags;                          /* flag-like options: See F_* defines in srl_decoder.c */
+    UV max_recursion_depth;             /* Configurable limit on the number of recursive calls we're willing to make */
     ptable_ptr ref_seenhash;            /* ptr table for avoiding circular refs */
     ptable_ptr ref_stashes;             /* ptr table for tracking stashes we will bless into - key: ofs, value: stash */
     ptable_ptr ref_bless_av;            /* ptr table for tracking which objects need to be bless - key: ofs, value: mortal AV (of refs)  */
     AV* weakref_av;
 
     UV bytes_consumed;
+    UV recursion_depth;                 /* Recursion depth of current decoder */
 } srl_decoder_t;
 
 /* constructor; don't need destructor, this sets up a callback */
